@@ -29,7 +29,6 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
-#include <gconf/gconf-client.h>
 #include <unique/uniqueapp.h>
 
 #include <bluetooth-client.h>
@@ -46,9 +45,10 @@ static GtkTreeModel *devices_model;
 static guint num_adapters_present = 0;
 static guint num_adapters_powered = 0;
 static gboolean show_icon_pref = TRUE;
+static gboolean discover_lock = FALSE;
 
-#define PREF_DIR		"/apps/bluetooth-manager"
-#define PREF_SHOW_ICON		PREF_DIR "/show_icon"
+#define SCHEMA_NAME		"org.gnome.Bluetooth"
+#define PREF_SHOW_ICON		"show-icon"
 
 #define KEYBOARD_PREFS		"gnome-keyboard-properties"
 #define MOUSE_PREFS		"gnome-mouse-properties"
@@ -62,7 +62,7 @@ enum {
 	DISCONNECTING
 };
 
-static GConfClient* gconf;
+static GSettings *settings;
 static BluetoothKillswitch *killswitch = NULL;
 
 static GtkBuilder *xml = NULL;
@@ -113,9 +113,16 @@ mount_finish_cb (GObject *source_object,
 
 	if (g_file_mount_enclosing_volume_finish (G_FILE (source_object),
 						  res, &error) == FALSE) {
-		g_printerr ("Failed to mount OBEX volume: %s", error->message);
-		g_error_free (error);
-		return;
+		/* Ignore "already mounted" error */
+		if (error->domain == G_IO_ERROR &&
+		    error->code == G_IO_ERROR_ALREADY_MOUNTED) {
+			g_error_free (error);
+			error = NULL;
+		} else {
+			g_printerr ("Failed to mount OBEX volume: %s", error->message);
+			g_error_free (error);
+			return;
+		}
 	}
 
 	uri = g_file_get_uri (G_FILE (source_object));
@@ -254,6 +261,45 @@ void wizard_callback(GObject *widget, gpointer user_data)
 		g_printerr("Couldn't execute command: %s\n", command);
 }
 
+static gboolean
+set_powered_foreach (GtkTreeModel *model,
+		     GtkTreePath  *path,
+		     GtkTreeIter  *iter,
+		     gpointer      data)
+{
+	DBusGProxy *proxy = NULL;
+	GValue value = { 0, };
+
+	gtk_tree_model_get (model, iter,
+			    BLUETOOTH_COLUMN_PROXY, &proxy, -1);
+	if (proxy == NULL)
+		return FALSE;
+
+	g_value_init (&value, G_TYPE_BOOLEAN);
+	g_value_set_boolean (&value, TRUE);
+
+	dbus_g_proxy_call_no_reply (proxy, "SetProperty",
+				    G_TYPE_STRING, "Powered",
+				    G_TYPE_VALUE, &value,
+				    G_TYPE_INVALID,
+				    G_TYPE_INVALID);
+
+	g_value_unset (&value);
+	g_object_unref (proxy);
+
+	return FALSE;
+}
+
+static void
+bluetooth_set_adapter_powered (void)
+{
+	GtkTreeModel *adapters;
+
+	adapters = bluetooth_client_get_adapter_model (client);
+	gtk_tree_model_foreach (adapters, set_powered_foreach, NULL);
+	g_object_unref (adapters);
+}
+
 void bluetooth_status_callback (GObject *widget, gpointer user_data)
 {
 	GObject *object;
@@ -265,6 +311,7 @@ void bluetooth_status_callback (GObject *widget, gpointer user_data)
 	bluetooth_killswitch_set_state (killswitch,
 					active ? KILLSWITCH_STATE_UNBLOCKED : KILLSWITCH_STATE_SOFT_BLOCKED);
 	g_object_set_data (object, "bt-active", GINT_TO_POINTER (active));
+	bluetooth_set_adapter_powered ();
 }
 
 void
@@ -273,7 +320,9 @@ bluetooth_discoverable_callback (GtkToggleAction *toggleaction, gpointer user_da
 	gboolean discoverable;
 
 	discoverable = gtk_toggle_action_get_active (toggleaction);
+	discover_lock = TRUE;
 	bluetooth_client_set_discoverable (client, discoverable);
+	discover_lock = FALSE;
 }
 
 static gboolean program_available(const char *program)
@@ -425,7 +474,7 @@ update_menu_items (void)
 	if (enabled == FALSE)
 		return;
 
-	gtk_action_set_sensitive (GTK_ACTION (object), TRUE);
+	gtk_action_group_set_sensitive (GTK_ACTION_GROUP (object), TRUE);
 }
 
 static void
@@ -452,6 +501,12 @@ update_discoverability (GtkTreeIter *iter)
 	gboolean discoverable;
 	GObject *object;
 
+	/* Avoid loops from changing the UI */
+	if (discover_lock != FALSE)
+		return;
+
+	discover_lock = TRUE;
+
 	object = gtk_builder_get_object (xml, "discoverable");
 
 	if (iter == NULL) {
@@ -465,6 +520,8 @@ update_discoverability (GtkTreeIter *iter)
 
 	gtk_action_set_visible (GTK_ACTION (object), TRUE);
 	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (object), discoverable);
+
+	discover_lock = FALSE;
 }
 
 static void
@@ -1006,20 +1063,13 @@ static void device_removed(GtkTreeModel *model,
 	device_changed (model, path, NULL, user_data);
 }
 
-static void gconf_callback(GConfClient *client, guint cnxn_id,
-					GConfEntry *entry, gpointer user_data)
+static void
+show_icon_changed (GSettings *settings,
+		   const char *key,
+		   gpointer   user_data)
 {
-	GConfValue *value;
-
-	value = gconf_entry_get_value(entry);
-	if (value == NULL)
-		return;
-
-	if (g_str_equal(entry->key, PREF_SHOW_ICON) == TRUE) {
-		show_icon_pref = gconf_value_get_bool(value);
-		update_icon_visibility();
-		return;
-	}
+	show_icon_pref = g_settings_get_boolean (settings, PREF_SHOW_ICON);
+	update_icon_visibility();
 }
 
 static GOptionEntry options[] = {
@@ -1032,7 +1082,6 @@ int main(int argc, char *argv[])
 	UniqueApp *app;
 	GtkStatusIcon *statusicon;
 	GtkWidget *menu;
-	GConfValue *value;
 	GOptionContext *context;
 	GError *error = NULL;
 
@@ -1092,20 +1141,11 @@ int main(int argc, char *argv[])
 					  bluetooth_killswitch_get_state (killswitch));
 	}
 
-	gconf = gconf_client_get_default();
+	settings = g_settings_new (SCHEMA_NAME);
+	show_icon_pref = g_settings_get_boolean (settings, PREF_SHOW_ICON);
 
-	value = gconf_client_get (gconf, PREF_SHOW_ICON, NULL);
-	if (value == NULL) {
-		show_icon_pref = TRUE;
-	} else {
-		show_icon_pref = gconf_value_get_bool (value);
-		gconf_value_free (value);
-	}
-
-	gconf_client_add_dir(gconf, PREF_DIR, GCONF_CLIENT_PRELOAD_NONE, NULL);
-
-	gconf_client_notify_add(gconf, PREF_DIR,
-					gconf_callback, NULL, NULL, NULL);
+	g_signal_connect (G_OBJECT (settings), "changed::" PREF_SHOW_ICON,
+			  G_CALLBACK (show_icon_changed), NULL);
 
 	statusicon = init_notification();
 
@@ -1122,7 +1162,7 @@ int main(int argc, char *argv[])
 
 	gtk_widget_destroy(menu);
 
-	g_object_unref(gconf);
+	g_object_unref(settings);
 
 	cleanup_agents();
 
