@@ -32,7 +32,7 @@
 #include <glib/gi18n.h>
 #include <math.h>
 #include <gdk/gdkkeysyms.h>
-#include <nbtk/nbtk-gtk.h>
+#include <mx/mx-gtk.h>
 
 #include "bluetooth-client.h"
 #include "bluetooth-client-private.h"
@@ -43,7 +43,6 @@
 #include "bluetooth-filter-widget.h"
 #include "bluetooth-agent.h"
 #include "gnome-bluetooth-enum-types.h"
-#include "bling-spinner.h"
 
 #include "pin.h"
 
@@ -72,6 +71,8 @@ struct _MoblinPanelPrivate
 	GtkWidget *label_pin;
 	GtkWidget *label_ssp_pin_help;
 	GtkWidget *label_ssp_pin;
+	GtkWidget *summary_frame;
+	GtkWidget *extra_config_vbox;
 	GtkWidget *label_failure;
 	GtkWidget *chooser;
 	GtkWidget *display;
@@ -121,6 +122,7 @@ typedef enum {
 	PAGE_SETUP,
 	PAGE_SSP_SETUP,
 	PAGE_CONNECTING,
+	PAGE_SUMMARY,
 	PAGE_FAILURE
 } MoblinPages;
 
@@ -139,7 +141,7 @@ static void update_random_pincode (MoblinPanel *self);
 static void create_selected_device (MoblinPanel *self);
 
 static void
-power_switch_toggled_cb (NbtkGtkLightSwitch *light_switch,
+power_switch_toggled_cb (MxGtkLightSwitch *light_switch,
 			 gboolean 	     state,
                          gpointer            user_data)
 {
@@ -166,6 +168,7 @@ enable_send_file (MoblinPanel *self)
 		guint i;
 		const char **uuids;
 
+		g_free (name);
 		bluetooth_chooser_get_selected_device_info (chooser, "uuids", &value);
 
 		uuids = (const char **) g_value_get_boxed (&value);
@@ -192,13 +195,13 @@ killswitch_state_changed_cb (BluetoothKillswitch *killswitch,
 	g_signal_handlers_block_by_func (priv->power_switch, power_switch_toggled_cb, user_data);
 
 	if (state == KILLSWITCH_STATE_SOFT_BLOCKED) {
-		nbtk_gtk_light_switch_set_active (NBTK_GTK_LIGHT_SWITCH (priv->power_switch),
+		mx_gtk_light_switch_set_active (MX_GTK_LIGHT_SWITCH (priv->power_switch),
 	                                      FALSE);
 		gtk_widget_set_sensitive (priv->power_switch, TRUE);
 		gtk_widget_set_sensitive (priv->add_new_button, FALSE);
 		gtk_widget_set_sensitive (priv->send_button, FALSE);
 	} else if (state == KILLSWITCH_STATE_UNBLOCKED) {
-		nbtk_gtk_light_switch_set_active (NBTK_GTK_LIGHT_SWITCH (priv->power_switch), TRUE);
+		mx_gtk_light_switch_set_active (MX_GTK_LIGHT_SWITCH (priv->power_switch), TRUE);
 		gtk_widget_set_sensitive (priv->power_switch, TRUE);
 		gtk_widget_set_sensitive (priv->add_new_button, TRUE);
 		enable_send_file (self);
@@ -243,7 +246,7 @@ send_file_button_clicked_cb (GtkButton *button,
 	GPtrArray *a;
 	GError *err = NULL;
 	guint i;
-	const char *address, *name;
+	char *address, *name;
 
 	address = bluetooth_chooser_get_selected_device (chooser);
 	name = bluetooth_chooser_get_selected_device_name (chooser);
@@ -264,6 +267,9 @@ send_file_button_clicked_cb (GtkButton *button,
 	}
 	g_ptr_array_add (a, NULL);
 
+	g_free (address);
+	g_free (name);
+
 	if (g_spawn_async(NULL, (char **) a->pdata, NULL,
 			  G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, &err) == FALSE) {
 		g_printerr("Couldn't execute command: %s\n", err->message);
@@ -462,14 +468,14 @@ remove_clicked_cb (GtkCellRenderer *cell, const gchar *path, gpointer user_data)
 	ensure_selection (chooser, path);
 
 	/* Get address */
-	if (bluetooth_chooser_get_selected_device_info (chooser, "address", &value)) {
-		address = g_value_get_string (&value);
-		g_value_unset (&value);
-	}
+	g_assert (bluetooth_chooser_get_selected_device_info (chooser, "address", &value));
+	address = g_value_get_string (&value);
 
 	if (bluetooth_chooser_remove_selected_device (chooser) != FALSE && address) {
 		bluetooth_plugin_manager_device_deleted (address);
 	}
+
+	g_value_unset (&value);
 }
 
 static void
@@ -486,7 +492,6 @@ browse_clicked (GtkCellRenderer *renderer, const gchar *path, gpointer user_data
 	/* Get address */
 	if (bluetooth_chooser_get_selected_device_info (chooser, "address", &value)) {
 		address = g_value_get_string (&value);
-		g_value_unset (&value);
 	}
 
 	if (address == NULL) {
@@ -496,6 +501,8 @@ browse_clicked (GtkCellRenderer *renderer, const gchar *path, gpointer user_data
 			g_printerr("Couldn't execute command: %s\n", cmd);
 		g_free (cmd);
 	}
+
+	g_value_unset (&value);
 }
 
 static void
@@ -604,21 +611,38 @@ connect_device (const gchar *device_path, MoblinPanel *self)
 	}
 }
 
+static void
+container_remove_all (GtkContainer *container)
+{
+	GList *list;
+	GList *iter;
+
+	list = gtk_container_get_children (container);
+	for (iter = list; iter; iter = iter->next)
+		gtk_container_remove (container, GTK_WIDGET (iter->data));
+
+	g_list_free (list);
+}
+
 static void
 set_current_page (MoblinPanel *self, MoblinPages page)
 {
 	MoblinPanelPrivate *priv = MOBLIN_PANEL_GET_PRIVATE (self);
 
 	if (page == PAGE_ADD) {
+		g_object_set (BLUETOOTH_CHOOSER (priv->chooser),
+			      "device-category-filter", BLUETOOTH_CATEGORY_NOT_PAIRED_OR_TRUSTED,
+			      NULL);
+
 		bluetooth_chooser_start_discovery (BLUETOOTH_CHOOSER (priv->chooser));
 	} else {
 		bluetooth_chooser_stop_discovery (BLUETOOTH_CHOOSER (priv->chooser));
 	}
 
 	if (page == PAGE_CONNECTING)
-		bling_spinner_start (BLING_SPINNER (priv->spinner));
+		gtk_spinner_start (GTK_SPINNER (priv->spinner));
 	else
-		bling_spinner_stop (BLING_SPINNER (priv->spinner));
+		gtk_spinner_stop (GTK_SPINNER (priv->spinner));
 
 	if ((page == PAGE_SETUP || page == PAGE_SSP_SETUP || page == PAGE_CONNECTING) && (priv->create_started == FALSE)) {
 		create_selected_device (self);
@@ -652,6 +676,45 @@ set_current_page (MoblinPanel *self, MoblinPages page)
 		gtk_widget_hide (priv->does_not_match_button);
 	}
 
+	if (page == PAGE_SUMMARY) {
+		GValue value = { 0, };
+		const char **uuids;
+		GList *widgets;
+		char *title;
+		gboolean have_additional_widgets = FALSE;
+
+		title = g_strdup_printf (_("Successfully set up new device '%s'"), priv->target_name);
+		set_frame_title (GTK_FRAME (priv->summary_frame), title);
+		g_free (title);
+
+		if (bluetooth_chooser_get_selected_device_info (BLUETOOTH_CHOOSER (priv->chooser), "uuids", &value))
+			uuids = g_value_get_boxed (&value);
+		else
+			uuids = NULL;
+
+		widgets = bluetooth_plugin_manager_get_widgets (priv->target_address, uuids);
+		if (widgets) {
+			GList *iter;
+
+			for (iter = widgets; iter; iter = iter->next)
+				gtk_box_pack_start (GTK_BOX (priv->extra_config_vbox),
+						    GTK_WIDGET (iter->data),
+						    FALSE,
+						    TRUE,
+						    0);
+
+			g_list_free (widgets);
+			gtk_widget_show_all (priv->extra_config_vbox);
+			have_additional_widgets = TRUE;
+		}
+
+		g_value_unset (&value);
+
+		if (!have_additional_widgets)
+			/* Show the devices page if we don't have any extra information to present. */
+			page = PAGE_DEVICES;
+	}
+
 	if (page == PAGE_DEVICES) {
 		/* Clean up old state */
 		update_random_pincode (self);
@@ -662,6 +725,7 @@ set_current_page (MoblinPanel *self, MoblinPages page)
 		priv->target_address = NULL;
 		g_free (priv->target_name);
 		priv->target_name = NULL;
+		container_remove_all (GTK_CONTAINER (priv->extra_config_vbox));
 		g_object_set (priv->chooser, "device-type-filter", BLUETOOTH_TYPE_ANY, NULL);
 	}
 
@@ -673,12 +737,14 @@ create_callback (BluetoothClient *client, const gchar *path, const GError *error
 {
 	MoblinPanel *self = MOBLIN_PANEL (user_data);
 	MoblinPanelPrivate *priv = MOBLIN_PANEL_GET_PRIVATE (self);
-	gchar *device_name = bluetooth_chooser_get_selected_device_name (BLUETOOTH_CHOOSER (priv->chooser));
+	gchar *device_name;
 
 	priv->create_started = FALSE;
 
 	if (path == NULL) {
+		device_name = bluetooth_chooser_get_selected_device_name (BLUETOOTH_CHOOSER (priv->chooser));
 		set_failure_message (self, device_name);
+		g_free (device_name);
 		set_current_page (self, PAGE_FAILURE);
 		return;
 	}
@@ -687,7 +753,7 @@ create_callback (BluetoothClient *client, const gchar *path, const GError *error
 
 	connect_device (path, self);
 
-	set_current_page (self, PAGE_DEVICES);
+	set_current_page (self, PAGE_SUMMARY);
 }
 
 static void
@@ -697,6 +763,12 @@ create_selected_device (MoblinPanel *self)
 	const gchar *path = AGENT_PATH;
 	gchar *pin_ret;
 
+	/* Set the filter on the selector, so we can use it to get more
+	 * info later, in page_summary */
+	g_object_set (priv->chooser,
+		      "device-category-filter", BLUETOOTH_CATEGORY_ALL,
+		      NULL);
+
 	pin_ret = get_pincode_for_device (priv->target_type, priv->target_address,
 					priv->target_name, NULL);
 	if (pin_ret != NULL && g_str_equal (pin_ret, "NULL"))
@@ -731,6 +803,7 @@ pair_clicked (GtkCellRenderer *renderer, const gchar *path, gpointer user_data)
 	}
 	if (bluetooth_chooser_get_selected_device_info (chooser, "legacypairing", &value) != FALSE) {
 		legacy_pairing = g_value_get_int (&value);
+		g_value_unset (&value);
 			if (legacy_pairing == -1)
 				legacy_pairing = TRUE;
 	} else {
@@ -738,10 +811,10 @@ pair_clicked (GtkCellRenderer *renderer, const gchar *path, gpointer user_data)
 	}
 
 	g_free (priv->target_address);
-	priv->target_address = g_strdup (address);
+	priv->target_address = address;
 
 	g_free (priv->target_name);
-	priv->target_name = g_strdup (name);
+	priv->target_name = name;
 
 	priv->target_type = type;
 	priv->target_ssp = !legacy_pairing;
@@ -1043,7 +1116,7 @@ create_failure_page (MoblinPanel *self)
 
 	priv = MOBLIN_PANEL_GET_PRIVATE (self);
 
-	page = nbtk_gtk_frame_new ();
+	page = mx_gtk_frame_new ();
 	page_title = gtk_label_new ("");
 	gtk_frame_set_label_widget (GTK_FRAME (page), page_title);
 	set_frame_title (GTK_FRAME (page), _("Device setup failed"));
@@ -1053,7 +1126,9 @@ create_failure_page (MoblinPanel *self)
 	gtk_widget_show (vbox);
 	gtk_container_add (GTK_CONTAINER (page), vbox);
 	priv->label_failure = gtk_label_new ("");
+	gtk_misc_set_alignment (GTK_MISC (priv->label_failure), 0.0, 0.5);
 	gtk_widget_show (priv->label_failure);
+	gtk_box_pack_start (GTK_BOX (vbox), priv->label_failure, FALSE, FALSE, 6);
 
 	hbox = gtk_hbox_new (FALSE, 6);
 	gtk_widget_show (hbox);
@@ -1066,6 +1141,45 @@ create_failure_page (MoblinPanel *self)
 	return page;
 }
 
+static GtkWidget *
+create_summary_page (MoblinPanel *self)
+{
+	MoblinPanelPrivate *priv;
+	GtkWidget *page;
+	GtkWidget *vbox, *hbox;
+	GtkWidget *w;
+
+	priv = MOBLIN_PANEL_GET_PRIVATE (self);
+
+	page = mx_gtk_frame_new ();
+	w = gtk_label_new (NULL);
+	gtk_frame_set_label_widget (GTK_FRAME (page), w);
+	gtk_widget_show (page);
+	priv->summary_frame = page;
+
+	vbox = gtk_vbox_new (FALSE, 12);
+	gtk_widget_show (vbox);
+	gtk_container_add (GTK_CONTAINER (page), vbox);
+	w = gtk_label_new (_("Select the additional services you want to use with your device:"));
+	gtk_misc_set_alignment (GTK_MISC (w), 0.0, 0.5);
+	gtk_box_pack_start (GTK_BOX (vbox), w, FALSE, FALSE, 6);
+	gtk_widget_show (w);
+
+	priv->extra_config_vbox = gtk_vbox_new (FALSE, 6);
+	gtk_widget_show (priv->extra_config_vbox);
+	gtk_box_pack_start (GTK_BOX (vbox), priv->extra_config_vbox, TRUE, TRUE, 6);
+
+	hbox = gtk_hbox_new (FALSE, 6);
+	gtk_widget_show (hbox);
+	gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, FALSE, 6);
+	w = gtk_button_new_with_label (_("Done"));
+	gtk_widget_show (w);
+	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 6);
+	g_signal_connect (w, "clicked", G_CALLBACK (set_device_view), self);
+
+	return page;
+}
+
 static GtkWidget *
 create_setup_page (MoblinPanel *self)
 {
@@ -1076,7 +1190,7 @@ create_setup_page (MoblinPanel *self)
 
 	priv = MOBLIN_PANEL_GET_PRIVATE (self);
 
-	page = nbtk_gtk_frame_new ();
+	page = mx_gtk_frame_new ();
 	page_title = gtk_label_new ("");
 	gtk_frame_set_label_widget (GTK_FRAME (page), page_title);
 	set_frame_title (GTK_FRAME (page), _("Device setup"));
@@ -1113,7 +1227,7 @@ create_ssp_setup_page (MoblinPanel *self)
 
 	priv = MOBLIN_PANEL_GET_PRIVATE (self);
 
-	page = nbtk_gtk_frame_new ();
+	page = mx_gtk_frame_new ();
 	page_title = gtk_label_new ("");
 	gtk_frame_set_label_widget (GTK_FRAME (page), page_title);
 	set_frame_title (GTK_FRAME (page), _("Device setup"));
@@ -1177,7 +1291,7 @@ create_add_page (MoblinPanel *self)
 	gtk_widget_show (vbox);
 	gtk_box_pack_start (GTK_BOX (page), vbox, TRUE, TRUE, 4);
 
-	frame = nbtk_gtk_frame_new ();
+	frame = mx_gtk_frame_new ();
 	frame_title = gtk_label_new ("");
 	gtk_frame_set_label_widget (GTK_FRAME (frame), frame_title);
 	set_frame_title (GTK_FRAME (frame), _("Devices"));
@@ -1213,7 +1327,7 @@ create_add_page (MoblinPanel *self)
 	gtk_box_pack_start (GTK_BOX (vbox), back_button, FALSE, FALSE, 4);
 
 	/* Right column */
-	frame = nbtk_gtk_frame_new ();
+	frame = mx_gtk_frame_new ();
 	gtk_widget_show (frame);
 	vbox = gtk_vbox_new (FALSE, 4);
 	gtk_widget_show (vbox);
@@ -1265,7 +1379,7 @@ create_devices_page (MoblinPanel *self)
 	gtk_widget_show (vbox);
 	gtk_box_pack_start (GTK_BOX (page), vbox, TRUE, TRUE, 4);
 
-	frame = nbtk_gtk_frame_new ();
+	frame = mx_gtk_frame_new ();
 	frame_title = gtk_label_new ("");
 	gtk_frame_set_label_widget (GTK_FRAME (frame), frame_title);
 	set_frame_title (GTK_FRAME (frame), _("Devices"));
@@ -1323,7 +1437,7 @@ create_devices_page (MoblinPanel *self)
 	gtk_box_pack_start (GTK_BOX (vbox), priv->add_new_button, FALSE, FALSE, 4);
 
 	/* Right column */
-	frame = nbtk_gtk_frame_new ();
+	frame = mx_gtk_frame_new ();
 	gtk_widget_show (frame);
 	vbox = gtk_vbox_new (FALSE, 4);
 	gtk_widget_show (vbox);
@@ -1342,7 +1456,7 @@ create_devices_page (MoblinPanel *self)
 	gtk_widget_show (power_label);
 	gtk_box_pack_start (GTK_BOX (hbox), power_label, FALSE, FALSE, 4);
 
-	priv->power_switch = nbtk_gtk_light_switch_new ();
+	priv->power_switch = mx_gtk_light_switch_new ();
 	if (priv->killswitch != NULL) {
 		if (bluetooth_killswitch_has_killswitches (priv->killswitch) == FALSE) {
 			gtk_widget_set_sensitive (priv->power_switch, FALSE);
@@ -1352,15 +1466,15 @@ create_devices_page (MoblinPanel *self)
 
 			switch (switch_state) {
 				case KILLSWITCH_STATE_UNBLOCKED:
-					nbtk_gtk_light_switch_set_active
-						(NBTK_GTK_LIGHT_SWITCH (priv->power_switch),
+					mx_gtk_light_switch_set_active
+						(MX_GTK_LIGHT_SWITCH (priv->power_switch),
 						 TRUE);
 					gtk_widget_set_sensitive (priv->add_new_button, TRUE);
 					gtk_widget_set_sensitive (priv->power_switch, TRUE);
 				break;
 				case KILLSWITCH_STATE_SOFT_BLOCKED:
-					nbtk_gtk_light_switch_set_active
-						(NBTK_GTK_LIGHT_SWITCH (priv->power_switch),
+					mx_gtk_light_switch_set_active
+						(MX_GTK_LIGHT_SWITCH (priv->power_switch),
 							FALSE);
 				break;
 				case KILLSWITCH_STATE_HARD_BLOCKED:
@@ -1406,7 +1520,7 @@ create_connecting_page (MoblinPanel *self)
 
 	priv = MOBLIN_PANEL_GET_PRIVATE (self);
 
-	page = nbtk_gtk_frame_new ();
+	page = mx_gtk_frame_new ();
 	page_title = gtk_label_new ("");
 	gtk_frame_set_label_widget (GTK_FRAME (page), page_title);
 	//set_frame_title (GTK_FRAME (page), _("Connecting"));
@@ -1415,7 +1529,7 @@ create_connecting_page (MoblinPanel *self)
 	vbox = gtk_vbox_new (FALSE, 6);
 	gtk_widget_show (vbox);
 	gtk_container_add (GTK_CONTAINER (page), vbox);
-	priv->spinner = bling_spinner_new ();
+	priv->spinner = gtk_spinner_new ();
 	gtk_widget_set_size_request (priv->spinner, 150, 150);
 	gtk_widget_show (priv->spinner);
 	gtk_box_pack_start (GTK_BOX (vbox), priv->spinner, FALSE, FALSE, 6);
@@ -1465,6 +1579,7 @@ moblin_panel_init (MoblinPanel *self)
 {
 	MoblinPanelPrivate *priv;
 	GtkWidget *devices_page, *add_page, *setup_page, *ssp_setup_page, *failure_page, *connecting_page;
+	GtkWidget *page;
 
 	priv = MOBLIN_PANEL_GET_PRIVATE (self);
 	priv->pin_dialog = NULL;
@@ -1522,6 +1637,10 @@ moblin_panel_init (MoblinPanel *self)
 	gtk_widget_show (connecting_page);
 	gtk_notebook_append_page (GTK_NOTEBOOK (priv->notebook), connecting_page, NULL);
 
+	page = create_summary_page (self);
+	gtk_widget_show (page);
+	gtk_notebook_append_page (GTK_NOTEBOOK (priv->notebook), page, NULL);
+
 	failure_page = create_failure_page (self);
 	gtk_widget_show (failure_page);
 	gtk_notebook_append_page (GTK_NOTEBOOK (priv->notebook), failure_page, NULL);
@@ -1538,9 +1657,20 @@ moblin_panel_dispose (GObject *object)
 
 	bluetooth_plugin_manager_cleanup ();
 
-	g_object_unref (priv->killswitch);
-	g_object_unref (priv->agent);
-	g_object_unref (priv->client);
+	if (priv->killswitch != NULL) {
+		g_object_unref (priv->killswitch);
+		priv->killswitch = NULL;
+	}
+
+	if (priv->agent != NULL) {
+		g_object_unref (priv->agent);
+		priv->agent = NULL;
+	}
+
+	if (priv->client != NULL) {
+		g_object_unref (priv->client);
+		priv->client = NULL;
+	}
 
 	G_OBJECT_CLASS (moblin_panel_parent_class)->dispose (object);
 }
