@@ -33,7 +33,6 @@
 #include <dbus/dbus-glib.h>
 #include <unique/uniqueapp.h>
 
-#include <bling-spinner.h>
 #include <bluetooth-client.h>
 #include <bluetooth-client-private.h>
 #include <bluetooth-chooser.h>
@@ -117,6 +116,7 @@ static GtkWidget *radio_auto = NULL;
 static GtkWidget *radio_0000 = NULL;
 static GtkWidget *radio_1111 = NULL;
 static GtkWidget *radio_1234 = NULL;
+static GtkWidget *radio_none = NULL;
 static GtkWidget *radio_custom = NULL;
 static GtkWidget *entry_custom = NULL;
 
@@ -505,7 +505,7 @@ void prepare_callback (GtkWidget *assistant,
 
 		complete = FALSE;
 
-		bling_spinner_start (BLING_SPINNER (spinner_connecting));
+		gtk_spinner_start (GTK_SPINNER (spinner_connecting));
 
 		/* translators:
 		 * The '%s' is the device name, for example:
@@ -515,7 +515,7 @@ void prepare_callback (GtkWidget *assistant,
 		gtk_label_set_text (GTK_LABEL (label_connecting), text);
 		g_free (text);
 	} else {
-		bling_spinner_stop (BLING_SPINNER (spinner_connecting));
+		gtk_spinner_stop (GTK_SPINNER (spinner_connecting));
 	}
 
 	if ((page == page_setup || page == page_connecting) && (create_started == FALSE)) {
@@ -564,7 +564,7 @@ void prepare_callback (GtkWidget *assistant,
 
 		complete = FALSE;
 
-		bling_spinner_start (BLING_SPINNER (spinner_finishing));
+		gtk_spinner_start (GTK_SPINNER (spinner_finishing));
 
 		/* translators:
 		 * The '%s' is the device name, for example:
@@ -574,7 +574,7 @@ void prepare_callback (GtkWidget *assistant,
 		gtk_label_set_text (GTK_LABEL (label_finishing), text);
 		g_free (text);
 	} else {
-		bling_spinner_stop (BLING_SPINNER (spinner_finishing));
+		gtk_spinner_stop (GTK_SPINNER (spinner_finishing));
 	}
 
 	if (page == page_summary) {
@@ -965,6 +965,7 @@ create_wizard (void)
 	radio_0000 = W("radio_0000");
 	radio_1111 = W("radio_1111");
 	radio_1234 = W("radio_1234");
+	radio_none = W("radio_none");
 	radio_custom = W("radio_custom");
 	entry_custom = W("entry_custom");
 
@@ -972,6 +973,7 @@ create_wizard (void)
 	g_object_set_data (G_OBJECT (radio_0000), "pin", "0000");
 	g_object_set_data (G_OBJECT (radio_1111), "pin", "1111");
 	g_object_set_data (G_OBJECT (radio_1234), "pin", "1234");
+	g_object_set_data (G_OBJECT (radio_none), "pin", "NULL");
 	g_object_set_data (G_OBJECT (radio_custom), "entry", entry_custom);
 
 	gtk_builder_connect_signals(builder, NULL);
